#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <math.h>
#include <stdlib.h>
#include <stdint.h>

#include "uart.h"

#define BAUD      9600UL
#define UBRR_BAUD   ((F_CPU/(8L*BAUD))-1)

// Globale Variablen
static char TxBuff[TX_BUFFER_SIZE];

static volatile uint8_t TxRead;		// Zeigt auf Zeichen, das gesendet werden soll
static uint8_t TxWrite;		// Zeigt auf nchste Schreibposition im Puffer

// Funktionen ffentlich
void InitUART(void) {
		UBRRH = (uint8_t) (UBRR_BAUD>>8);
	UBRRL = (uint8_t) (UBRR_BAUD & 0xFF);

	UCSRB=(1<<RXEN)|(1<<TXEN);
	UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);

}

void UartPutC(char data) {
	while(TxWrite != TxRead);								// Warte bis Platz im Puffer
	TxBuff[TxWrite++] = data;
	if(TxWrite >= TX_BUFFER_SIZE) TxWrite = 0;
	UCSRB |= (1<<UDRIE);									// UDRE Interrupt ein
}

void UartPutS(const char *s) {
	while(*s) UartPutC(*s++);
}


char Ausgabe(signed int zahl, int stelle)
{ 
	int temp=abs(zahl);
	temp=temp/pow(10,stelle); 
	temp=temp-temp/10*10; 
	return((temp+48));
}


void print(signed int zahl)
{
	int x;
	if (zahl<0)
	{ 
		UartPutC('-');
	}
	for( x=-1; x<=3; x++)
	{	 
		UartPutC(Ausgabe(zahl,3-x));
	}
}



void UartPutP(const char *progmem_s) {
	register char c;
	while((c = pgm_read_byte(progmem_s++))) UartPutC(c);
}

// Interrupt Service Routinen
ISR(USART_UDRE_vect) {
	if(TxRead == TxWrite) UCSRB &= ~(1<<UDRIE);			// UDRE Interrupt aus
	else {
		UDR = TxBuff[TxRead++];							// nchtes Zeichen aus Puffer senden
		if(TxRead >= TX_BUFFER_SIZE) TxRead = 0;
	} 							
}


unsigned char USART_Receive( void )
{
/* Wait for data to be received */
while ( !(UCSRA & (1<<RXC)) )
;
/* Get and return received data from buffer */
return UDR;
}

